include LinodeApi

def load_current_resource
  @current_resource = Chef::Resource::LinodeDnsRecord.new(new_resource.name)
  @current_resource.domain(new_resource.domain)
  @current_resource.type(new_resource.type)
  @current_resource.domain_id(lookup_domain_id(new_resource.domain))

  params = {
    :DomainID => @current_resource.domain_id,
    :Type     => @current_resource.type,
    :Name     => @current_resource.name,
  }

  linode_record = get_record(params)
  Chef::Log.info "Got current Linode DNS record #{linode_record.inspect} for #{params.inspect}"

  unless linode_record.nil?
    @current_resource.target(linode_record.target)
    @current_resource.port(linode_record.port)
    @current_resource.priority(linode_record.priority)
    @current_resource.protocol(linode_record.protocol)
    @current_resource.linode_id(linode_record.resourceid)
    @current_resource.ttl(linode_record.ttl_sec)
    @current_resource.weight(linode_record.weight)
  end

  @current_resource
end

action :create do
  domain = new_resource.domain
  params = {
    :DomainID => current_resource.domain_id,
    :Type      => new_resource.type,
  }
  params[:Name]       = new_resource.name      unless new_resource.name.nil?
  params[:Target]     = new_resource.target    unless new_resource.target.nil?
  params[:Priority]   = new_resource.priority  unless new_resource.priority.nil?
  params[:Weight]     = new_resource.weight    unless new_resource.weight.nil?
  params[:Port]       = new_resource.port      unless new_resource.port.nil?
  params[:Protocol]   = new_resource.protocol  unless new_resource.protocol.nil?
  params[:TTL_sec]    = new_resource.ttl       unless new_resource.ttl.nil?

  params[:ResourceID] = current_resource.linode_id unless current_resource.linode_id.nil?

  if params[:ResourceID].nil?
    Chef::Log.info "Creating new Linode DNS record for #{params.inspect}"
    new_record = linode_api.domain.resource.create(params)

    new_resource.linode_id(new_record.resourceid)

    if dns_record_exists?(params)
      new_resource.updated_by_last_action(true)
    else
      raise "Unable to create DNS record for #{params.inspect}; result was: #{new_record.inspect}"
    end
  elsif dns_record_exists?(params)
    Chef::Log.info "Not creating nor updating Linode DNS record; exact match exists"
    new_resource.updated_by_last_action(false)
  else
    Chef::Log.info "Updating existing Linode DNS record for #{params.inspect}"
    updated_record = linode_api.domain.resource.update(params)
    if dns_record_exists?(params)
      new_resource.updated_by_last_action(true)
    else
      raise "Unable to update DNS record for #{params.inspect}; result was: #{updated_record.inspect}"
    end
  end
end

action :delete do
  Chef::Log.info "Deleting Linode DNS record for #{params.inspect}"
  if current_resource.domain_id.nil? || current_resource.linode_id.nil?
    raise "No Linode DNS record matching #{new_resource.inspect} found"
  end
  result = linode_api.domain.resource.delete(:DomainId => current_resource.domain_id,
                                             :ResourceId => current_resource.linode_id)

  if dns_record_exists?(:domain_id => current_resource.domain_id,
                        :resource_id => current_resource.linode_id)
    raise "Unable to delete DNS record for #{new_resource.name}.#{new_resource.domain}; result was: #{result.inspect}"
  else
    new_resource.updated_by_last_action(true)
  end
end
